import 'package:flutter/material.dart';

// InteractivePolygon returns a Polygon object wrapped in a GestureDetector
// in order to make the Polygon clickable
class InteractivePolygon extends StatelessWidget {
  final List<Offset> points;
  final VoidCallback onTap;
  final Color color;
  final double strokeWidth;

  InteractivePolygon({
    required this.points,
    required this.onTap,
    this.color = Colors.blue,
    this.strokeWidth = 1.0,
  });

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: onTap,
      child: CustomPaint(
        painter: PolygonPainter(
          points: points,
          color: color,
          strokeWidth: strokeWidth,
        ),
      ),
    );
  }
}

// PolygonPainter takes the points, color, and stroke width from a
// object of type InteractivePolygon, and renders it
class PolygonPainter extends CustomPainter {
  final List<Offset> points;
  final Color color;
  final double strokeWidth;

  PolygonPainter({
    required this.points,
    required this.color,
    required this.strokeWidth,
  });

  // paint() iterates through all the vertices of the polygon and connects
  // each sequential pair with a line
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()
      ..color = color
      ..style = PaintingStyle.stroke
      ..strokeWidth = strokeWidth;

    final path = Path();
    path.moveTo(points[0].dx, points[0].dy);
    for (var i = 1; i < points.length; i++) {
      path.lineTo(points[i].dx, points[i].dy); // Render line between vertices
    }
    path.close();

    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(PolygonPainter oldDelegate) {
    return oldDelegate.points != points ||
        oldDelegate.color != color ||
        oldDelegate.strokeWidth != strokeWidth;
  }
}